#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define SE stderr
#define PI 3.141592654
#define EPS 0.000001
#define MAX 19

int xs,ys,xf,yf; // line segment
int aa,bb,cc,dd; // rotation into the correct octant

FILE *F;

/* ================================ 
 * SUPPLEMENTARY CODE
 * ================================ */


void WritePSHeader(void){
  fprintf(F,"%%!PS (c) MisoF\n\n");
  
  fprintf(F,"%% ---------\n%% definicie\n%% ---------\n\n");
  fprintf(F,"/cm {72 2.54 div mul} def\n/pis {newpath moveto show} def\n");
  fprintf(F,"/C { gsave newpath 0 360 arc gsave setgray fill grestore\n");
  fprintf(F,"     setgray setlinewidth stroke grestore } bind def\n\n");

  fprintf(F,"%% --------\n%% hlavicka\n%% --------\n\n");
  fprintf(F,"/Times-Roman findfont 24 scalefont setfont\n");
  fprintf(F,"(2D grafika - homework #3) 174 800 pis\n\n");

  fprintf(F,"/Times-Roman findfont 16 scalefont setfont\n1.4 setlinewidth\n\n");
  fprintf(F,"(\\(c\\) MisoF) 520 820 pis\n\n");

  fprintf(F,"%% ----\n%% siet\n%% ----\n\n");
  fprintf(F,"30 30 570 { 30 30 570 {\n");
  fprintf(F," 2 copy pop exch 1 0 1 5 copy pop pop pop exch 10 C pop pop\n");
  fprintf(F,"} for } for\n\n");
}

/* ================================
 * MAIN FUNCTIONS
 * ================================ */

void PutPixel(int xx, int yy){
  int x,y;
  x=aa*xx+bb*yy; y=cc*xx+dd*yy;
  printf("[%d,%d] ",x,y);
//  fprintf(F,"1 0 0.4 %d %d 10 C\n",30*x,30*y);
  fprintf(F,"1 0 0.4 %d %d 10 C\n",30*x,30*y);
}

void DrawLine(int xs, int ys, int xf, int yf){
fprintf(F,"1.0 0.0 0.0 setrgbcolor\n");
fprintf(F,"newpath %d %d moveto %d %d lineto stroke\n",30*xs,30*ys,30*xf,30*yf);
fprintf(F,"0.0 0.0 0.0 setrgbcolor\n");
fprintf(F,"(Line: [%d,%d]-[%d,%d]) 20 600 pis\n",xs,ys,xf,yf);
}

void Bresenham(int oxs, int oys, int oxf, int oyf) {
  int dx,dy,p,kdex,kdey,inc1,inc2;
  int xs,ys,xf,yf;

  // trafo do oktantu so sklonom z <0,1>
  xs=oxs; xf=oxf; ys=oys; yf=oyf;
  if (xf>=xs) {
    if (yf>=ys) {
      if (abs(xf-xs)>=abs(yf-ys)){ aa=1; bb=0; cc=0; dd=1;
      } else { aa=xs; xs=ys; ys=aa; aa=xf; xf=yf; yf=aa;
      aa=0; bb=1; cc=1; dd=0; }
    } else {
      ys=-ys; yf=-yf;
      if (abs(xf-xs)>=abs(yf-ys)){ aa=1; bb=0; cc=0; dd=-1;
      } else { aa=xs; xs=ys; ys=aa; aa=xf; xf=yf; yf=aa;
      aa=0; bb=1; cc=-1; dd=0; }
    }
  } else {
    xs=-xs; xf=-xf; 
    if (yf>=ys) {
      if (abs(xf-xs)>=abs(yf-ys)){ aa=-1; bb=0; cc=0; dd=1;
      } else { aa=xs; xs=ys; ys=aa; aa=xf; xf=yf; yf=aa;
      aa=0; bb=-1; cc=1; dd=0; }
    } else {
      ys=-ys; yf=-yf;
      if (abs(xf-xs)>=abs(yf-ys)){ aa=-1; bb=0; cc=0; dd=-1;
      } else { aa=xs; xs=ys; ys=aa; aa=xf; xf=yf; yf=aa;
      aa=0; bb=-1; cc=-1; dd=0; }
    }
  }
  // end trafo
  dx=xf-xs; dy=yf-ys; p=2*dy-dx; inc1=2*dy-2*dx; inc2=2*dy;
  kdex=xs; kdey=ys;
  PutPixel(kdex,kdey);
  while (kdex<xf) {
    kdex++; if (p>0) { kdey++; p+=inc1; } else { p+=inc2; }
    PutPixel(kdex,kdey);
  }
  printf("\n");
  DrawLine(oxs,oys,oxf,oyf);
}
    
int main(int argc, char **argv){
  fprintf(SE,"2D Grafika - Domaca uloha #3 v1.0\n(c) Miso Forisek\n");
  if (argc>1){
    if (strcmp(argv[1],"--help")==0) {
      fprintf(SE,"\nProgram nepodporuje ziadne parametre okrem --help.\n");
      fprintf(SE,"\nSyntax vstupu:\n");
      fprintf(SE,"x1, y1, x2, y2 -- suradnice koncovych bodov usecky\n");
      fprintf(SE,"                  (cele cisla z intervalu 1..%d)\n\n",MAX);
      fprintf(SE,"Program vykresli usecku a jej aproximaciu v subore hw3.ps\n");
      return 0;
    } else {
      fprintf(SE,"Neznamy parameter. Pouzi --help\n");
      return 0;
    }
  }

  F=fopen("hw3.ps","wt");
  
  scanf("%d %d %d %d ",&xs,&ys,&xf,&yf);
  if ((xs<1)||(xs>MAX)||(ys<1)||(ys>MAX)||(xf<1)||(xf>MAX)||(yf<1)||(yf>MAX)){
    printf("Coordinates are not in the range 1..%d!\n",MAX);
    return 1;
  }

  WritePSHeader();

  Bresenham(xs,ys,xf,yf);

  fprintf(F,"showpage\n");
  fclose(F);

  return 0;
}

