#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define SE stderr
#define PI 3.141592654
#define EPS 0.000001
#define MAXN 1005

typedef struct { double ymin,ymax,x,m1; } tGELrec;
typedef struct { double curx; int no; } tAELrec;

FILE *F,*inF;
int i,N;
double Poly[MAXN][2];
tGELrec GEL[MAXN];
tAELrec AEL[MAXN];
int inGEL,inAEL,sweepy,sweepend;

/* ================================ 
 * SUPPLEMENTARY CODE
 * ================================ */

int CMP(double a, double b){
  if (a<b-EPS) return -1;
  if (b<a-EPS) return 1;
  return 0;
}

int GCMP(const void *a, const void *b){
  tGELrec *aa,*bb;
  aa=(tGELrec *)a; bb=(tGELrec *)b;
  if ((aa->ymin)<(bb->ymin)) return -1;
  if ((bb->ymin)<(aa->ymin)) return 1;
  if ((aa->x)<(bb->x)) return -1;
  if ((bb->x)<(aa->x)) return 1;
  return 0;
}

int ACMP(const void *a, const void *b){
  tAELrec *aa,*bb;
  aa=(tAELrec *)a; bb=(tAELrec *)b;
  if ((aa->curx)<(bb->curx)) return -1;
  if ((bb->curx)<(aa->curx)) return 1;
  return 0;
}

void WritePSHeader(void){
  fprintf(F,"%%!PS (c) MisoF\n\n");
  
  fprintf(F,"%% ---------\n%% definicie\n%% ---------\n\n");
  fprintf(F,"/cm {72 2.54 div mul} def\n/pis {newpath moveto show} def\n");

  fprintf(F,"/Stvorec { gsave 0 0 0.75 setrgbcolor ");
  fprintf(F,"15 mul 60 add exch 15 mul 60 add exch\n");
  fprintf(F,"newpath 2 copy moveto 2 copy 15 add lineto 2 copy 15 add exch\n");
  fprintf(F,"15 add exch lineto 2 copy exch 15 add exch lineto closepath fill\n");
  fprintf(F,"0 setgray\n");
  fprintf(F,"newpath 2 copy moveto 2 copy 15 add lineto 2 copy 15 add exch\n");
  fprintf(F,"15 add exch lineto 2 copy exch 15 add exch lineto closepath stroke\n");
  fprintf(F,"grestore } bind def\n\n");
  
//  fprintf(F,"/C { gsave newpath 0 360 arc gsave setgray fill grestore\n");
//  fprintf(F,"     setgray setlinewidth stroke grestore } bind def\n\n");

  fprintf(F,"%% --------\n%% hlavicka\n%% --------\n\n");
  fprintf(F,"/Times-Roman findfont 24 scalefont setfont\n");
  fprintf(F,"(2D grafika - homework #5) 174 800 pis\n\n");

  fprintf(F,"/Times-Roman findfont 16 scalefont setfont\n1.4 setlinewidth\n\n");
  fprintf(F,"(\\(c\\) MisoF) 520 820 pis\n\n");

  fprintf(F,"%% ----\n%% siet\n%% ----\n\n");
  fprintf(F,"75 15 525 {\n");
  fprintf(F,"newpath dup 75 moveto dup 525 lineto stroke\n");
  fprintf(F,"newpath dup 75 exch moveto 525 exch lineto stroke\n");
  fprintf(F,"} for\n\n");
  
//  fprintf(F,"%% ----\n%% siet\n%% ----\n\n");
//  fprintf(F,"30 30 570 { 30 30 570 {\n");
//  fprintf(F," 2 copy pop exch 1 0 1 5 copy pop pop pop exch 10 C pop pop\n");
//  fprintf(F,"} for } for\n\n");
}

/* ================================
 * MAIN FUNCTIONS
 * ================================ */

void PutPixel(int xx, int yy){
//  x=aa*xx+bb*yy; y=cc*xx+dd*yy;
//   printf("[%d,%d] ",x,y);
  fprintf(F,"%d %d Stvorec\n",xx,yy);
}

void DrawLine(int xs, int ys, int xf, int yf){
  fprintf(F,"1.0 0.0 0.0 setrgbcolor\n");
  fprintf(F,"newpath %.1f %.1f moveto %.1f %.1f lineto stroke\n",
          67.5+15.0*xs,67.5+15.0*ys,67.5+15.0*xf,67.5+15.0*yf);
  fprintf(F,"0.0 0.0 0.0 setrgbcolor\n");
//  fprintf(F,"(Line: [%d,%d]-[%d,%d]) 20 600 pis\n",xs,ys,xf,yf);
}

void InitGEL(void){
  int i,j;
  double x1,y1,x2,y2;

  inGEL=inAEL=0;
  sweepy=10000; sweepend=0;

  for (i=1;i<=N;i++){
    if (Poly[i][1]<sweepy) sweepy=Poly[i][1];
    if (Poly[i][1]>sweepend) sweepend=Poly[i][1];
    
    if (CMP(Poly[i][1],Poly[i+1][1])==0) {
      // vykreslime
      y1=Poly[i][1];
      x1=Poly[i][0];
      if (CMP(Poly[i+1][0],x1)==-1) { x2=x1; x1=Poly[i+1][0]; } else x2=Poly[i+1][0];
      for (j=(int)(ceil(x1));j<=floor(x2);j++) PutPixel(j,(int)(y1));
      
      continue;
    } 
    if (CMP(Poly[i][1],Poly[i+1][1])==-1) {
      y1=Poly[i][1]; y2=Poly[i+1][1]; x1=Poly[i][0]; x2=Poly[i+1][0];
      GEL[inGEL].m1=(x2-x1)/(y2-y1);
      if ((Poly[i-1][1]-Poly[i][1])*(Poly[i][1]-Poly[i+1][1])>=0) {
        x1+=GEL[inGEL].m1; y1+=1;
      }
    } else {
      y1=Poly[i+1][1]; y2=Poly[i][1]; x1=Poly[i+1][0]; x2=Poly[i][0];
      GEL[inGEL].m1=(x2-x1)/(y2-y1);
      if ((Poly[i][1]-Poly[i+1][1])*(Poly[i+1][1]-Poly[i+2][1])>=0) {
        x1+=GEL[inGEL].m1; y1+=1;
      }
    }
    
    GEL[inGEL].ymin=y1; GEL[inGEL].ymax=y2; GEL[inGEL].x=x1; 
    inGEL++;
  }

  // zotriedime GEL
  qsort(GEL,inGEL,sizeof(tGELrec),GCMP);

//  for (i=0;i<inGEL;i++) printf("y1=%.2f y2=%.2f x1=%.2f m1=%f \n",GEL[i].ymin,GEL[i].ymax,GEL[i].x,GEL[i].m1);
  
  sweepy--; sweepend++; 
}

void ScanLineConversion(void){
  int wGEL,i,j;
  
  InitGEL();
  wGEL=0; GEL[inGEL].ymin=sweepend+10;
 
  for (;sweepy<=sweepend;sweepy++){
    // updatneme hrany v AEL
    for (i=0;i<inAEL;i++) AEL[i].curx+=GEL[ AEL[i].no ].m1;
    // pridame hrany do AEL
    while (CMP(GEL[wGEL].ymin,sweepy)<=0) {
      AEL[inAEL].no=wGEL;
      AEL[inAEL].curx=GEL[wGEL].x;
      inAEL++; wGEL++;
    }
    // uberieme hrany z AEL
    for (i=0;i<inAEL;) { 
      if (CMP(GEL[ AEL[i].no ].ymax,sweepy)==-1){
         AEL[i].no=AEL[inAEL-1].no;
         AEL[i].curx=AEL[inAEL-1].curx;
         inAEL--;
      } else i++;
    }
    // utriedime AEL
    qsort(AEL,inAEL,sizeof(tAELrec),ACMP);
    // vypiseme
    printf("Sweeping line: %d\n",sweepy);
    for (i=0;i<inAEL;i++) printf("[x=%.2f no=%d] ",AEL[i].curx,AEL[i].no);
    printf("\n");
    // vykreslime
    if (inAEL%2==1) { fprintf(SE,"Sth rotten!\n"); fclose(F); exit(1); }
    for (i=0;i<(inAEL/2);i++) {
      for (j=(int)ceil(AEL[2*i].curx);j<=floor(AEL[2*i+1].curx);j++)
        PutPixel(j,sweepy);
    }
  }
}

int main(int argc, char **argv){
  fprintf(SE,"2D Grafika - Domaca uloha #5 v1.0\n(c) Miso Forisek\n");
  if (((argc>1)&&(strcmp(argv[1],"--help")==0))||(argc==1)){
    fprintf(SE,"Pouzitie:\n%s",argv[0]);
    fprintf(SE," [subor_s_popisom_polygonu]\n\n");
           
    fprintf(SE,"Subor s popisom polygonu ma mat nasledovnu strukturu:\n");
    fprintf(SE,"  N -- pocet vrcholov polygonu\n");
    fprintf(SE,"  N dvojic celych cisel z <0,30> -- suradnice vrcholov po obvode.\n\n");
    
    fprintf(SE,"Program vytvori subor hw5.ps s grafickym znazornenim vystupu.\n");
    return 0;
  }

  F=fopen("hw5.ps","wt");
  inF=fopen(argv[1],"rt");
  fscanf(inF,"%d ",&N);
  for (i=0;i<N;i++) fscanf(inF,"%lf %lf ",&Poly[i][0],&Poly[i][1]);
  Poly[N][0]=Poly[0][0]; Poly[N][1]=Poly[0][1];
  Poly[N+1][0]=Poly[1][0]; Poly[N+1][1]=Poly[1][1];
  Poly[N+2][0]=Poly[2][0]; Poly[N+2][1]=Poly[2][1];
  fclose(inF);
  
/*  scanf("%d %d %d %d ",&xs,&ys,&xf,&yf);
  if ((xs<1)||(xs>MAX)||(ys<1)||(ys>MAX)||(xf<1)||(xf>MAX)||(yf<1)||(yf>MAX)){
    printf("Coordinates are not in the range 1..%d!\n",MAX);
    return 1;
  }*/

  WritePSHeader();
//  PutPixel(3,5);
//  DrawLine(3,5,6,6);

  ScanLineConversion();
  for (i=0;i<N;i++) DrawLine((int)Poly[i][0],(int)Poly[i][1],(int)Poly[i+1][0],(int)Poly[i+1][1]);

  fprintf(F,"showpage\n");
  fclose(F);

  return 0;
}

